#!/bin/sh
#
#	/etc/rc.d/init.d
#
# Automatic/manual config of DRAC4 virtual UART and startup of DRAC4 Service
#
# chkconfig: 2345 20 80
# description: Configures/starts the host connection to the DRAC4 controller
# processname: racsvc
# config: /etc/sysconfig/racsvc.conf

# source common function library
#. /etc/rc.d/init.d/functions

#Changes by DTK are the following
#1. Create /dev/ttyS? files
#2. remove all references to rh rc scripts(success() and failure())
#3. set path to point to rac binaries located in /opt/dell/toolkit/bin
#4. Change the variable racsvc to racsv4 so that we can have racsvc version
#   3 and 4 on the same machine.

# create serial device files
mknod /dev/ttyS0 c 4 64 >/dev/null 2>&1
mknod /dev/ttyS1 c 4 65 >/dev/null 2>&1
mknod /dev/ttyS2 c 4 66 >/dev/null 2>&1
mknod /dev/ttyS3 c 4 67 >/dev/null 2>&1
mknod /dev/ttyS4 c 4 68 >/dev/null 2>&1
mknod /dev/ttyS5 c 4 69 >/dev/null 2>&1
# and so on......

export PATH=$PATH:/opt/dell/toolkit/bin

program=`basename $0`
service="racsvc"
cfgfile="/etc/sysconfig/${service}.conf"

LANG=C		# runs in 'C' locale
RETVAL=0

# source service configuration file, if available
[ -f ${cfgfile} ] && . ${cfgfile}

racsvc="Dell DRAC4 Service"
racpci="1028:0012"
emsg=
norac="DRAC4 pci device not found"
notty="DRAC4 tty device not found"
maddr=
irqno=
ttyid=

# probe for the DRAC4 pci device. if found, save DRAC4 IRQ and I/O ADDR.
function testrac
{
	which setserial >/dev/null 2>&1
	[ $? -ne 0 ] && { emsg="'setserial' not found"; return 2; }
	irqno=`lspci -d ${racpci} -v 2>/dev/null|grep "IRQ " 2>/dev/null`
	[ -z "${irqno}" ] && { emsg=${norac}; return 3; }
	irqno=${irqno##* }
	maddr=`lspci -d ${racpci} -v 2>/dev/null|grep "I/O " 2>/dev/null`
	[ -z "${maddr}" ] && { emsg=${norac}; return 3; }
	maddr=${maddr% [*}
	maddr=${maddr##* }
	uaddr=`echo ${maddr}|tr [a-f] [A-F]`
	laddr=`echo ${maddr}|tr [A-F] [a-f]`
	return 0
}

# find an available tty, or the one already in use by DRAC4
function get_ttyname
{
	# ensure that DRAC4 is present
	testrac || return $?

	# check whether the DRAC4 device is already in use by serial
	tag="port"
	sinfo="/proc/tty/driver/serial"
	hit=`cat ${sinfo}|grep -v unknown|grep "${tag}:[0]*${laddr}\|${tag}:[0]*${uaddr}"`

	if [ -n "$hit" ]; then
		ttyname="ttyS"`echo $hit|cut -f1 -d':'`
	else
		# DRAC4 port not yet in use; pick lowest available ttyno >= 2
		hit=99999
		shopt -s extglob
		for i in `ls /dev/ttyS+([[:digit:]])`
		{
			ttyno=${i#/dev/ttyS}
			if [ ${ttyno} -ge 2 -a ${ttyno} -lt $hit ]; then
				bzy=`cat ${sinfo} 2>/dev/null|grep -v unknown|grep "^${ttyno}:"`
				[ -z "${bzy}" ] && hit=${ttyno}
			fi
		}
		[ $hit -eq 99999 ] && { emsg=${notty}; return 2; }	# none found
		tag=
		ttyname="ttyS$hit"
	fi

	# check for pre-configured tty name
	if [ -n "$DRAC4_TTY" -a ! "$DRAC4_TTY" = "$ttyname" ]; then

		# tty for DRAC4 device is being changed, 'unset' prior tty
		[ -n "$tag" ] && setserial /dev/${ttyname} -z \
											uart unknown port 0x0 irq 0x0
		ttyname=${DRAC4_TTY}
	fi

	return 0
}

svcpfx="${service} -d /dev/"
# find out if service is running
racbzy=`ps --no-headers -C ${service} -o"%a"|grep "^${svcpfx}"`

# handle request
case "$1" in
  start)
	if [ -n "${racbzy}" ]; then
		echo -n "${racsvc} already running... "
		RETVAL=0		# be nicer
	else
		get_ttyname
		RETVAL=$?
		if [ $RETVAL -ne 0 ]; then
			echo -n "DRAC4 probe: ${emsg}"
		else
			echo -n "Starting ${racsvc}... "
			if [ ! -c /dev/${ttyname} ]; then
				echo -n "invalid tty: '${ttyname}'"
				RETVAL=4
			fi
			
			# first clear the serial flags, etc. for the chosen DRAC4 tty.
			# without this, we may get screaming interrupts.
			setserial /dev/${ttyname} -z uart unknown port 0x0 irq 0x0
			
			# set serial characteristics for DRAC4 device
			# this depends on kernel version (due to serial rewrite in 2.6)
			kver=`uname -r|cut -d'.' -f1`
			krel=`uname -r|cut -d'.' -f2`
			knum=$((kver*100 + krel))
			if [ $knum -lt 206 ]; then # -- pre-2.6 kernel
				setserial /dev/${ttyname} \
					port 0x${maddr} irq ${irqno} ^skip_test autoconfig
				setserial /dev/${ttyname} \
					uart 16550A low_latency baud_base 1382400	\
					close_delay 0 closing_wait infinite
				stty -F /dev/${ttyname} crtscts raw
			else	# -- 2.6+ kernel
				setserial /dev/${ttyname} \
					port 0x${maddr} irq ${irqno} ^skip_test autoconfig
				setserial /dev/${ttyname} \
					uart 16550A low_latency baud_base 1382400	\
					close_delay 0 closing_wait infinite
				stty -F /dev/${ttyname} crtscts raw \
					-echo -echoctl -echoe -echok -echoke -echonl -echoprt
			fi

			# start the rac service
			${svcpfx}${ttyname} $RACSVC_OPTS >/dev/null 2>&1
			[ $? -ne 0 ] && RETVAL=8
		fi
	fi
	echo
	;;
  stop)
	echo -n "Stopping ${racsvc}... "
	if [ -z "${racbzy}" ]; then
		echo -n "not running "
		RETVAL=1
	else
		racsvc -s >/dev/null 2>&1
		[ $? -ne 0 ] && RETVAL=8
	fi
	echo
	;;
  status)
	echo -n "Checking ${racsvc}... " 
	if [ -z "${racbzy}" ]; then
		echo -n "not running "
		RETVAL=1
	fi
	echo
	;;
  probe)
	echo -n "DRAC4 probe: "	
	testrac
	RETVAL=$?
	[ -n "${emsg}" ] && echo -n "${emsg} "
	echo
	;;
  *)
	echo "Usage: ${service} {start|stop|status|probe}"
	RETVAL=1
esac

exit $RETVAL
